__title__ = 'i3ipc'
__description__ = 'An improved Python library to control i3wm and sway'
__url__ = 'https://github.com/altdesktop/i3ipc-python'
__version__ = '2.2.1'
__author__ = 'Tony Crisci'
__author_email__ = 'tony@dubstepdish.com'
__license__ = 'BSD-3-Clause'
__copyright__ = 'Copyright 2015 Tony Crisci'
